<?php
/*
WCMS
Simple file based CMS that supports inclusion of html, xhtml, php and cgi files.
The CMS is controlled with one file per page and a number of control files:
- index.php, wraps it all up
- menu, contains references to pages
- users, contains users and passwords

WCMS is written with a procedural paradigm. The file index.php can be adapted to 
one's wishes. It can contain the following functions.
- parseQuery();
- parseSession();
- parseMenu();
- parseContent();
- printMainMenu();
- printMenu();
- printContent();

*/

// mode of menulinks is either 'ugly', 'rewrite', 'session' or 'javascript'
// It indicates how the current page is communicated to index.php
// in ugly mode it is in $_GET['name'], in rewrite mode is is in $_SERVER{"REDIRECT_URL"}
// in session mode it is $_SESSION['name'] unless it is in $_GET['name']
// in javascript mode it is in $_GET['name'] and only the content part is returned.
// the mode also afects what the menu links to.
$mode = 'ugly'; 

function parseQuery($prefix=""){
	// parse the query, 
	// $requestFile is the requested file either with relative path from this script location or absolute path.
	// $queryString is the queryString that is passed to the requested file
	// $scriptFilePath is the directory of this script as seen in the file system
	// $scriptUrlDir is the directory of this script as seen in the address bar of the browser

	global $requestFile;
	global $scriptUrlDir;
	global $scriptUrlPrefix;
	$scriptUrlPrefix = $prefix; 
	global $queryString;

	global $mode;

	// $scriptUrlPrefix is a prefix in the url that was removed by .htaccess
	if(strlen($scriptUrlPrefix) || isset($_SERVER["REDIRECT_URL"])){
		$mode = 'rewrite';
	}

	$requestFile = "";
	$queryString = "";
	if(isset($GLOBALS["argv"])){ // command line
		if(sizeof($GLOBALS["argv"])>1)
			$requestFile = $GLOBALS["argv"][1];
		if(sizeof($GLOBALS["argv"])>2)
			$queryString = $GLOBALS["argv"][2];
		$scriptUrlDir = "";
	} elseif(isset($_SERVER['SCRIPT_FILENAME'])){ // CGI
		$scriptUrlDir = dirname($_SERVER['SCRIPT_NAME']); 
		if ($scriptUrlDir=='/') $scriptUrlDir = ""; # dirnames should never end in /, not even for root
		if($mode=='ugly'){
			if(isset($_GET['file']))
				$requestFile= $_GET['file'];
		} elseif($mode=='rewrite') {
			// get the requested filename from REQUEST_URI, this is not changed by RewriteRule
			if(preg_match("|^$scriptUrlDir/$scriptUrlPrefix(.*?)(\?(.*))?$|", $_SERVER['REQUEST_URI'], $matches)){
				$requestFile = $matches[1]; # filename in original request (before rewriting)
				if(count($matches)>3)
					$queryString = $matches[3];
			} else
				$requestFile = "500";
		}	else
			$requestFile = "500";
	} else {
		$requestFile = '500';
	}
	return array($requestFile, $scriptUrlDir, $queryString);
}

function parseSession($fileName){
	# all session stuff prior to sending out the http header.
	if(!isset($_SESSION)) 
		session_start();

	## LOGOUT
	# we never end a session, we merely make it a 'non login session'
	if(isset($_SESSION['user']) && isset($_REQUEST['logout'])){
		session_unset(); 
	}

	## LOGIN
	elseif(!isset($_SESSION['user']) && isset($_REQUEST['user']) && isset($_REQUEST['password'])){
		# calculate hash value to put in user file for this password
		if($_REQUEST['user']=='md5'){
			$_SESSION['error'] = md5($_REQUEST['password']);
			return;
		}
		if($userArray = file($fileName)){
			foreach ($userArray as $line){
				list($level, $user, $md5Password) = explode(' ', rtrim($line));
				if($user == $_REQUEST['user'] && $md5Password == md5($_REQUEST['password'])){
					$_SESSION['user'] = $user;
					$_SESSION['level'] = $level;
					return array($user, $level);
				}
			}
			$_SESSION['error'] = "Access denied.";
		} else
			$_SESSION['error'] = "Userlist not available.";
	}
}

function file2url($file, $mode){
	global $scriptUrlDir;
	global $scriptUrlPrefix;
	if($mode == 'rewrite'){
		if(!preg_match("|://|", $file) && !preg_match("|^/|", $file)) # not an absolute link
			return sprintf("%s/%s%s", $scriptUrlDir, $scriptUrlPrefix, $file);
		else
			return $file;
	} elseif($mode == 'ugly'){
		if(!preg_match("|://|", $file) && !preg_match("|^/|", $file)) # not an absolute link
			return sprintf("%s?file=%s", $_SERVER['SCRIPT_NAME'], "$scriptUrlPrefix$file");
		else
			return sprintf("%s?file=%s", $_SERVER['SCRIPT_NAME'], $file);
	}
}

function parseMenu($fileName, $requestFile){
	// parse the menu
	// menuActive i, menuActiveToplevel i, menuLength, i, 
	// menuBranch[b], menuOpen[i], menuVisible[i]
	// menuFile[s], menuName[s], menuUrl[s], menuDepth[i], menuLevel[i]
	global $mode;
	global $scriptUrlDir;
	global $scriptUrlPrefix;
	global $queryString;
	global $menu;
	
	$menuArray = file($fileName);
	$menuActive = -1; 
	$menuLength = sizeof($menuArray);
	$menuBranch = array_fill(0, $menuLength, false);
	$iItemAllowed = 0; # allowed menu item (<= $i)
	for ($i=0; $i<$menuLength; $i++) {
		$line = chop($menuArray[$i]);
		if(strspn($line, "1234567890") != 1)
			continue;
		$level = (int)substr($line, 0, 1); # userlevel
		if($level>0 && (!isset($_SESSION['level']) || $_SESSION['level']<$level))
			continue;
		$menuLevel[] = $level;
		$depth = strspn($line, " ", 2) / 2; # menu depth
		$line = substr($line, 2+2*$depth);
		$menuVisible[] = $depth==0; # items that are always visible
		$menuDepth[] = $depth; # number of spaces after first two character / 2
		$pos = strpos($line, " "); # position of space after 'file'
		if ($pos) { # there is a file
			$file = substr($line, 0, $pos);
			$name = substr($line, $pos+1);
		} else { # there is no file
			$file = sprintf("menuItem%03d.html", $i);
			$name = substr($line, $depth);
		}

		$menuUrl[] = file2url($file, $mode);
		
		$menuFile[] = $file;
		$menuName[] = $name;
		if($file == $requestFile){ # match
			$menuActive = $iItemAllowed;
			$menuFilename = $menuFile[$iItemAllowed];
		}
		if($i>0 && $menuDepth[$iItemAllowed-1]<$depth) # previous menu item was a branch, not a leaf
			$menuBranch[$iItemAllowed-1] = true;
		$iItemAllowed++;
	}
	$menuLength = sizeof($menuDepth);
	

	# find corresponding depth=0 menu
	$menuActiveToplevel = -1;
	for ($i=$menuActive; $i>=0; $i--){
		if($menuDepth[$i]==0){
			$menuActiveToplevel = $i;
			break;
		}
	}

	// set menuVisible and menuOpen
	$menuOpen = array_fill(0, sizeof($menuFile), false);
	if($menuActive >= 0){ # there is an active menu
		# make list of open/visible menus, first upward, then downward
		$largestDepth = $menuDepth[$menuActive]+1; #largest depth of a menu that must be open
		for ($i = $menuActive; $i>=0; $i--) {
			if($menuDepth[$i]<$largestDepth){ # always true for menuActive
				$menuOpen[$i] = $menuBranch[$i];
				$menuVisible[$i] = true;
				if($largestDepth>0)
					$largestDepth--;
			} elseif($menuDepth[$i]==$largestDepth){
				$menuVisible[$i] = true;
			}
		}
		$largestDepth = $menuDepth[$menuActive];
		$openMenus = true;
		$visibleMenus = true;
		for ($i = $menuActive+1; $i<sizeof($menuFile); $i++) {
			$depth = $menuDepth[$i];
			if($depth==$largestDepth)
				$openMenus = false;
			if($depth<$largestDepth)
				$visibleMenus = false;
			if(($openMenus && $depth==$largestDepth+1) || ($visibleMenus && $depth==$largestDepth) || $depth==0)
				$menuVisible[$i] = true;
		}
	}	elseif(strcmp($requestFile, "")==0){ # first menu item is default, choose it if nothing was requested
		$menuFilename = $menuFile[0];
		$menuActive = 0;
	} else { # there is a request, but it is not in the menu
		if(preg_match("|\d{3}|", $requestFile))
			$menuFilename = sprintf("error/%03d.html", $requestFile); # injection risk, be careful when changing this line
		elseif(file_exists($requestFile))
			$menuFilename = "error/403.html";
		else
			$menuFilename = "error/404.html";
	}


	// build return structure
	$menu['active'] = $menuActive;                 // int
	$menu['activeToplevel'] = $menuActiveToplevel; // int
	$menu['branch'] = $menuBranch;                 // bool[length]
	$menu['open'] = $menuOpen;                     // bool[length]
	$menu['visible'] = $menuVisible;               // bool[length]
	$menu['file'] = $menuFile;                     // string[length]
	$menu['name'] = $menuName;                     // string[length]
	$menu['url'] = $menuUrl;                       // string[length]
	$menu['depth'] = $menuDepth;                   // int[length]
	$menu['level'] = $menuLevel;                   // int[length]
	$menu['filename'] = $menuFilename;             // string, for safety disconnected from $requestFile
	return $menu;
}

function parseContent($menu){
	// alter environment to mimick non-CMS
	global $mode;
	switch ($mode){
	case 'rewrite':
		$_SERVER{"SCRIPT_NAME"} = $_SERVER{"REDIRECT_URL"};
		$_SERVER{"PHP_SELF"} = $_SERVER{"REDIRECT_URL"};
		break;
	case 'ugly':
		// if REDIRECT_QUERY_STRING:	'file=testArgs.php' else QUERY_STRING:  'file=testArgs.php'
		#$_SERVER{"QUERY_STRING"} = substr($_SERVER{"REQUEST_URI"}, strrpos($_SERVER{"REQUEST_URI"}, "?")+1);
		$_SERVER{"SCRIPT_NAME"} .= "?file=${_GET['file']}"; # breaks down when using forms
	}

	// read the content and put it in $content
	$content = "";
	$title = "";
	$style = "";
	$content .= "<div class='right'>" . $menu['filename'] . "</div>\n"; // show filename
	if(preg_match('|menuItem\d{3}|', $menu['filename'], $matches)){ # menu only, no content
		$content .= sprintf("<H1>%s</H1>\n", $menu['name'][$menu['active']]);
		$title = $menu['name'][$menu['active']];
		$depth = $menu['depth'][$menu['active']];
		$i = $menu['active'];
		for($i=$menu['active']+1; $i<count($menu['depth']) && $menu['depth'][$i]>$menu['depth'][$menu['active']]; $i++) {
			$depth = $menu['depth'][$i]-$menu['depth'][$menu['active']];
			if($i==$menu['active']+1 or $menu['depth'][$i]>$menu['depth'][$i-1])
				$content .=  str_repeat("	", $depth-1)."<ul>\n";
			if($i!=$menu['active']+1 and $menu['depth'][$i]<$menu['depth'][$i-1])
				$content .=  str_repeat("	", $depth)."</ul>\n";
			$content .=  str_repeat("	", $depth)."<li><a href='{$menu['url'][$i]}'>{$menu['name'][$i]}</a></li>\n";
		}
		while($depth>=$menu['depth'][$menu['active']])
			$content .=  str_repeat("	", $depth--)."</ul>\n";
	} else { # file
		if(preg_match('|\.pl$|', $menu['filename'], $matches)){ #cgi
			# big injection risk, be careful here
			$env = "";
			foreach ($_SERVER  as $key => $value)
				if(preg_match('|^[A-Z_]+$|', $key))
					$env .= "$key=".escapeshellarg($value)." ";
			# cwd
			$dir = dirname($_SERVER['SCRIPT_FILENAME']);
			error_log("boe: $dir");

			if($_SERVER['REQUEST_METHOD'] == 'GET')
				$file = `PERL5LIB=/home/wilberth/public_html/cgi-bin $env $dir/${menu['filename']}`; #execute
			else {
				$post = escapeshellarg(http_build_query($_POST));
				$env .= 'PERL5LIB=/home/wilberth/public_html/cgi-bin ';
				$file = `export $env; echo $post | $dir/${menu['filename']}`; #execute
			}
		} else # txt, php or html
			$file = file_get_contents($menu['filename']);
		if(preg_match('|\.txt$|', $menu['filename'])){
			$file = preg_replace('|<|', '&lt;', $file);
			$file = preg_replace('|>|', '&gt;', $file);
			$file = preg_replace('|^(.*?)\n|', '<h1>$1</h1>', $file);
			$file = preg_replace('|\n\n|', '<p>', $file);
		}
		if($file){ # file contents exists
			# title
			if(preg_match('|<head>.*<title>(.*?)</title>.*</head>|si', $file, $matches))
				$title = $matches[1];
			else {
				preg_match('|<h1>(.*?)</h1>|si', $file, $matches);
				$title = $matches[1];
			}
			# only body
			if(preg_match('|<body>(.*)</body>|si', $file, $matches)){
				$content .= $matches[1];
				if(preg_match('|<head>.*(<style(.*?)</style>).*</head>|si', $file, $matches))
				$style = $matches[1];
			} else
				$content .= $file;
				#include('test.php');
		} else
			$content .= file_get_contents('error/404.html');
	}
	return array($content, $style, $title);
}



function printMainMenu($menu){
	for ($i=0; $i<count($menu['depth']); $i++)
		if($menu['depth'][$i]==0)
			printf("\t\t\t<li><a %shref='%s'>%s</a></li>\n", 
				$i==$menu['activeToplevel']?"class='current' ":"", $menu['url'][$i], $menu['name'][$i]);
}

function printMenu($menu){
	for ($i=0; $i<count($menu['depth']); $i++)
		if($menu['visible'][$i])
			printf("\t\t\t<a class='nav%s%s%s sub%d' href='%s'>%s</a><span class='hide'> | </span>\n", 
				$i==$menu['active']?" active":"", 
				$menu['branch'][$i]?" branch":"", 
				$menu['open'][$i]?" open":"", 
				$menu['depth'][$i], $menu['url'][$i], $menu['name'][$i]);
				#$menuBranch[$i]?"(b)":"", $menuOpen[$i]?"(o)":"", $menuVisible[$i]?"(v)":"");
}

function printSession(){
	// print either a notice that a user was logged on or a login form
	if(isset($_SESSION['error'])){
		printf("<p class='searchform'><b>%s</b></p>", $_SESSION['error']);
		unset($_SESSION['error']);
	}
	if(isset($_SESSION['user'])){
		printf("<p class='searchform'>user: %s (<a href='%s?logout=1'>logout</a>)</p>", 
			$_SESSION['user'], $_SERVER['PHP_SELF']);
	} else 
		# note that PHP_SELF has already been changed to the contained page
		printf('
			<form action="%s" method="post">
			<p class="searchform">
				u:<input type="text" alt="user" name="user" class="loginbox"/><br/>
				p:<input type="password" alt="password" name="password" class="loginbox"/>
				<input type="submit" value="Go!" class="searchbutton" />
			</p>
			</form>
			', $_SERVER["PHP_SELF"]);
}

function printContent($content){
	if(FALSE === eval(" ?>$content<?php ")) 
		print "$content";
}

function printSitemap(){
	global $menu;
	print "<ul>\n";
	$currentDepth = 0;
	for($i=0; $i<sizeof($menu['depth']); $i++) {
		$depth = $menu['depth'][$i];
		while($currentDepth < $depth)
			print(str_repeat("	", ++$currentDepth)."<li style='list-style-type:none'><ul>\n");
		while($currentDepth > $depth)
			print(str_repeat("	", $currentDepth--)."</ul></li>\n");
		print(str_repeat("	", $depth)."<li><a href='{$menu['url'][$i]}'>{$menu['name'][$i]}</a></li>\n");
	}
	while($depth>=0)
		print (str_repeat("	", $depth--)."</ul>\n");
	print "</ul>\n";
}

?>
